==> rust-preserves@0.1.2.zip: preserves/implementations/rust/src/value/de.rs <==
use crate::value::{Value, AValue, Dictionary};
use crate::value::value::{Float, Double};
use crate::value::error::{Error, Result, ExpectedKind};
use num::traits::cast::ToPrimitive;
use serde::Deserialize;
use serde::de::{Visitor, SeqAccess, MapAccess, EnumAccess, VariantAccess, DeserializeSeed};
use std::convert::TryFrom;
use std::iter::Iterator;

pub struct Deserializer<'de> {
    input: &'de AValue,
}

pub fn from_value<'a, T>(v: &'a AValue) -> Result<T> where T: Deserialize<'a> {
    let mut de = Deserializer::from_value(v);
    let t = T::deserialize(&mut de)?;
    Ok(t)
}

impl<'de> Deserializer<'de> {
    pub fn from_value(v: &'de AValue) -> Self {
        Deserializer{ input: v }
    }

    fn check<'a, T, F>(&'a mut self, f: F, k: ExpectedKind) -> Result<T>
    where F: FnOnce(&'de Value) -> Option<T>
    {
        f(self.input.value()).ok_or_else(|| Error::Expected(k, self.input.clone()))
    }
}

impl<'de, 'a> serde::de::Deserializer<'de> for &'a mut Deserializer<'de> {
    type Error = Error;

    fn deserialize_any<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let v = self.input.value();
        match v {
            Value::Boolean(b) => visitor.visit_bool(*b),
            Value::Float(Float(f)) => visitor.visit_f32(*f),
            Value::Double(Double(d)) => visitor.visit_f64(*d),
            Value::SignedInteger(ref i) =>
                match i.to_i64() {
                    None => match i.to_u64() {
                        None => Err(Error::NumberTooLarge(i.clone())),
                        Some(n) => visitor.visit_u64(n),
                    }
                    Some(n) => visitor.visit_i64(n),
                }
            Value::String(ref s) => visitor.visit_str(&s),
            Value::ByteString(_) => self.deserialize_bytes(visitor),
            Value::Record(_) =>
                if v.is_simple_record("tuple", Some(0)) {
                    self.deserialize_unit(visitor)
                } else if v.is_simple_record("UnicodeScalar", Some(1)) {
                    self.deserialize_char(visitor)
                } else
                    if v.is_simple_record("None", Some(0)) ||
                    v.is_simple_record("Some", Some(1))
                {
                    self.deserialize_option(visitor)
                } else if v.is_simple_record("tuple", None) {
                    visitor.visit_seq(VecSeq::new(self, v.as_simple_record("tuple", None).unwrap()))
                } else {
                    Err(Error::CannotDeserializeAny)
                }
            Value::Sequence(ref v) => visitor.visit_seq(VecSeq::new(self, v)),
            Value::Dictionary(ref d) => visitor.visit_map(DictMap::new(self, d)),
            _ => Err(Error::CannotDeserializeAny),
        }
    }

    fn deserialize_bool<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_bool(self.check(|v| v.as_boolean(), ExpectedKind::Boolean)?)
    }

    fn deserialize_i8<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i8(i.to_i8().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i16<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i16(i.to_i16().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i32(i.to_i32().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i64(i.to_i64().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u8<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u8(i.to_u8().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u16<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u16(i.to_u16().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u32(i.to_u32().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u64(i.to_u64().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_f32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_f32(self.check(|v| v.as_float(), ExpectedKind::Float)?)
    }

    fn deserialize_f64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_f64(self.check(|v| v.as_double(), ExpectedKind::Double)?)
    }

    fn deserialize_char<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record("UnicodeScalar", Some(1)),
                            ExpectedKind::SimpleRecord("UnicodeScalar", Some(1)))?;
        let c = fs[0].value().as_u32()
            .ok_or_else(|| Error::Expected(ExpectedKind::SignedInteger, self.input.clone()))?;
        visitor.visit_char(char::try_from(c).or(Err(Error::InvalidUnicodeScalar(c)))?)
    }

    fn deserialize_str<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_str(&self.check(|v| v.as_string(), ExpectedKind::String)?)
    }

    fn deserialize_string<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        self.deserialize_str(visitor)
    }

    fn deserialize_bytes<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_bytes(&self.check(|v| v.as_bytestring(), ExpectedKind::ByteString)?)
    }

    fn deserialize_byte_buf<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_byte_buf(self.check(|v| v.as_bytestring(), ExpectedKind::ByteString)?.clone())
    }

    fn deserialize_option<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        match self.input.value().as_simple_record("None", Some(0)) {
            Some(_fs) => visitor.visit_none(),
            None => match self.input.value().as_simple_record("Some", Some(1)) {
                Some(fs) => {
                    self.input = &fs[0];
                    visitor.visit_some(self)
                }
                None => Err(Error::Expected(ExpectedKind::Option, self.input.clone()))
            }
        }
    }

    fn deserialize_unit<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        if self.input.value().is_simple_record("tuple", Some(0)) {
            visitor.visit_unit()
        } else {
            Err(Error::Expected(ExpectedKind::SimpleRecord("tuple", Some(0)), self.input.clone()))
        }
    }

    fn deserialize_unit_struct<V>(self, name: &'static str, visitor: V)
                                  -> Result<V::Value> where V: Visitor<'de>
    {
        if self.input.value().is_simple_record(name, Some(0)) {
            visitor.visit_unit()
        } else {
            Err(Error::Expected(ExpectedKind::SimpleRecord(name, Some(0)), self.input.clone()))
        }
    }

    fn deserialize_newtype_struct<V>(self, name: &'static str, visitor: V)
                                     -> Result<V::Value> where V: Visitor<'de>
    {
        if name == crate::value::value::MAGIC {
            let mut buf: Vec<u8> = Vec::new();
            crate::value::Encoder::new(&mut buf, None).write(self.input)
                .or_else(|_e| Err(Error::InternalMagicError))?;
            visitor.visit_byte_buf(buf)
        } else {
            let fs = self.check(|v| v.as_simple_record(name, Some(1)),
                                ExpectedKind::SimpleRecord(name, Some(1)))?;
            self.input = &fs[0];
            visitor.visit_newtype_struct(self)
        }
    }

    fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        let vs = self.check(|v| v.as_sequence(), ExpectedKind::Sequence)?;
        visitor.visit_seq(VecSeq::new(self, vs))
    }

    fn deserialize_tuple<V>(self, len: usize, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record("tuple", Some(len)),
                            ExpectedKind::SimpleRecord("tuple", Some(len)))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_tuple_struct<V>(self, name: &'static str, len: usize, visitor: V)
                                   -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record(name, Some(len)),
                            ExpectedKind::SimpleRecord(name, Some(len)))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_map<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        let d = self.check(|v| v.as_dictionary(), ExpectedKind::Dictionary)?;
        visitor.visit_map(DictMap::new(self, d))
    }

    fn deserialize_struct<V>(self,
                             name: &'static str,
                             fields: &'static [&'static str],
                             visitor: V)
                             -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record(name, Some(fields.len())),
                            ExpectedKind::SimpleRecord(name, Some(fields.len())))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_enum<V>(self,
                           _name: &'static str,
                           _variants: &'static [&'static str],
                           visitor: V)
                           -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_enum(self)
    }

    fn deserialize_identifier<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_str(&self.check(|v| v.as_symbol(), ExpectedKind::Symbol)?)
    }

    fn deserialize_ignored_any<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_none()
    }
}

pub struct VecSeq<'a, 'de: 'a> {
    index: usize,
    vec: &'de Vec<AValue>,
    de: &'a mut Deserializer<'de>,
}

impl<'de, 'a> VecSeq<'a, 'de> {
    fn new(de: &'a mut Deserializer<'de>, vec: &'de Vec<AValue>) -> Self {
        VecSeq{ index: 0, vec, de }
    }
}

impl<'de, 'a> SeqAccess<'de> for VecSeq<'a, 'de> {
    type Error = Error;

    fn next_element_seed<T>(&mut self, seed: T)
                            -> Result<Option<T::Value>> where T: DeserializeSeed<'de>
    {
        if self.index == self.vec.len() {
            return Ok(None)
        }

        self.de.input = &self.vec[self.index];
        self.index = self.index + 1;
        let value = seed.deserialize(&mut *self.de)?;
        Ok(Some(value))
    }
}

pub struct DictMap<'a, 'de: 'a> {
    pending: Option<&'de AValue>,
    iter: Box<dyn Iterator<Item = (&'de AValue, &'de AValue)> + 'a>,
    de: &'a mut Deserializer<'de>,
}

impl<'de, 'a> DictMap<'a, 'de> {
    fn new(de: &'a mut Deserializer<'de>, d: &'de Dictionary) -> Self {
        DictMap{ pending: None, iter: Box::new(d.into_iter()), de }
    }
}

impl<'de, 'a> MapAccess<'de> for DictMap<'a, 'de> {
    type Error = Error;

    fn next_key_seed<K>(&mut self, seed: K)
                        -> Result<Option<K::Value>> where K: DeserializeSeed<'de>
    {
        match self.iter.next() {
            None => Ok(None),
            Some((k, v)) => {
                self.pending = Some(v);
                self.de.input = k;
                Ok(Some(seed.deserialize(&mut *self.de)?))
            }
        }
    }

    fn next_value_seed<V>(&mut self, seed: V) -> Result<V::Value> where V: DeserializeSeed<'de> {
        let v = self.pending.unwrap();
        self.pending = None;
        self.de.input = v;
        Ok(seed.deserialize(&mut *self.de)?)
    }
}

impl<'a, 'de> EnumAccess<'de> for &'a mut Deserializer<'de> {
    type Error = Error;
    type Variant = Self;

    fn variant_seed<V>(self, seed: V)
                       -> Result<(V::Value, Self::Variant)> where V: DeserializeSeed<'de>
    {
        let (lp, _) = self.check(|v| v.as_record(), ExpectedKind::Record)?;
        let v = self.input;
        self.input = lp;
        let variant = seed.deserialize(&mut *self)?;
        self.input = v;
        Ok((variant, self))
    }
}

impl<'a, 'de> VariantAccess<'de> for &'a mut Deserializer<'de> {
    type Error = Error;

    fn unit_variant(self) -> Result<()> {
        Ok(())
    }

    fn newtype_variant_seed<T>(self, seed: T) -> Result<T::Value> where T: DeserializeSeed<'de> {
        let (_, fs) = self.input.value().as_record().unwrap();
        self.input = &fs[0];
        seed.deserialize(&mut *self)
    }

    fn tuple_variant<V>(self, _len: usize, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        visitor.visit_seq(VecSeq::new(self, &self.input.value().as_record().unwrap().1))
    }

    fn struct_variant<V>(self, _fields: &'static [&'static str], visitor: V)
                         -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_seq(VecSeq::new(self, &self.input.value().as_record().unwrap().1))
    }
}

==> rust-preserves@0.2.2.zip: preserves/implementations/rust/src/value/de.rs <==
use crate::value::{Value, NestedValue, Map};
use crate::value::value::{Float, Double};
use crate::value::error::{Error, Result, ExpectedKind};
use num::traits::cast::ToPrimitive;
use serde::Deserialize;
use serde::de::{Visitor, SeqAccess, MapAccess, EnumAccess, VariantAccess, DeserializeSeed};
use std::convert::TryFrom;
use std::iter::Iterator;

pub struct Deserializer<'de, N: NestedValue> {
    input: &'de N,
}

pub fn from_value<'a, T, N: NestedValue>(v: &'a N) -> Result<T> where T: Deserialize<'a> {
    let mut de = Deserializer::from_value(v);
    let t = T::deserialize(&mut de)?;
    Ok(t)
}

impl<'de, N: NestedValue> Deserializer<'de, N> {
    pub fn from_value(v: &'de N) -> Self {
        Deserializer{ input: v }
    }

    fn check<'a, T, F>(&'a mut self, f: F, k: ExpectedKind) -> Result<T>
    where F: FnOnce(&'de Value<N>) -> Option<T>
    {
        f(self.input.value()).ok_or_else(|| Error::Expected(k, self.input.copy_via()))
    }
}

impl<'de, 'a, N: NestedValue> serde::de::Deserializer<'de> for &'a mut Deserializer<'de, N> {
    type Error = Error;

    fn deserialize_any<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let v = self.input.value();
        match v {
            Value::Boolean(b) => visitor.visit_bool(*b),
            Value::Float(Float(f)) => visitor.visit_f32(*f),
            Value::Double(Double(d)) => visitor.visit_f64(*d),
            Value::SignedInteger(ref i) =>
                match i.to_i64() {
                    None => match i.to_u64() {
                        None => Err(Error::NumberTooLarge(i.clone())),
                        Some(n) => visitor.visit_u64(n),
                    }
                    Some(n) => visitor.visit_i64(n),
                }
            Value::String(ref s) => visitor.visit_str(&s),
            Value::ByteString(_) => self.deserialize_bytes(visitor),
            Value::Record(_) =>
                if v.is_simple_record("tuple", Some(0)) {
                    self.deserialize_unit(visitor)
                } else if v.is_simple_record("UnicodeScalar", Some(1)) {
                    self.deserialize_char(visitor)
                } else
                    if v.is_simple_record("None", Some(0)) ||
                    v.is_simple_record("Some", Some(1))
                {
                    self.deserialize_option(visitor)
                } else if v.is_simple_record("tuple", None) {
                    visitor.visit_seq(VecSeq::new(self, v.as_simple_record("tuple", None).unwrap()))
                } else {
                    Err(Error::CannotDeserializeAny)
                }
            Value::Sequence(ref v) => visitor.visit_seq(VecSeq::new(self, v)),
            Value::Dictionary(ref d) => visitor.visit_map(DictMap::new(self, d)),
            _ => Err(Error::CannotDeserializeAny),
        }
    }

    fn deserialize_bool<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_bool(self.check(|v| v.as_boolean(), ExpectedKind::Boolean)?)
    }

    fn deserialize_i8<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i8(i.to_i8().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i16<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i16(i.to_i16().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i32(i.to_i32().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_i64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_i64(i.to_i64().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u8<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u8(i.to_u8().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u16<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u16(i.to_u16().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u32(i.to_u32().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_u64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let i = self.check(|v| v.as_signedinteger(), ExpectedKind::SignedInteger)?;
        visitor.visit_u64(i.to_u64().ok_or(Error::NumberTooLarge(i.clone()))?)
    }

    fn deserialize_f32<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_f32(self.check(|v| v.as_float(), ExpectedKind::Float)?)
    }

    fn deserialize_f64<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_f64(self.check(|v| v.as_double(), ExpectedKind::Double)?)
    }

    fn deserialize_char<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record("UnicodeScalar", Some(1)),
                            ExpectedKind::SimpleRecord("UnicodeScalar", Some(1)))?;
        let c = fs[0].value().as_u32()
            .ok_or_else(|| Error::Expected(ExpectedKind::SignedInteger, self.input.copy_via()))?;
        visitor.visit_char(char::try_from(c).or(Err(Error::InvalidUnicodeScalar(c)))?)
    }

    fn deserialize_str<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_str(&self.check(|v| v.as_string(), ExpectedKind::String)?)
    }

    fn deserialize_string<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        self.deserialize_str(visitor)
    }

    fn deserialize_bytes<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_bytes(&self.check(|v| v.as_bytestring(), ExpectedKind::ByteString)?)
    }

    fn deserialize_byte_buf<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_byte_buf(self.check(|v| v.as_bytestring(), ExpectedKind::ByteString)?.clone())
    }

    fn deserialize_option<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        match self.input.value().as_simple_record("None", Some(0)) {
            Some(_fs) => visitor.visit_none(),
            None => match self.input.value().as_simple_record("Some", Some(1)) {
                Some(fs) => {
                    self.input = &fs[0];
                    visitor.visit_some(self)
                }
                None => Err(Error::Expected(ExpectedKind::Option, self.input.copy_via()))
            }
        }
    }

    fn deserialize_unit<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        if self.input.value().is_simple_record("tuple", Some(0)) {
            visitor.visit_unit()
        } else {
            Err(Error::Expected(ExpectedKind::SimpleRecord("tuple", Some(0)), self.input.copy_via()))
        }
    }

    fn deserialize_unit_struct<V>(self, name: &'static str, visitor: V)
                                  -> Result<V::Value> where V: Visitor<'de>
    {
        if self.input.value().is_simple_record(name, Some(0)) {
            visitor.visit_unit()
        } else {
            Err(Error::Expected(ExpectedKind::SimpleRecord(name, Some(0)), self.input.copy_via()))
        }
    }

    fn deserialize_newtype_struct<V>(self, name: &'static str, visitor: V)
                                     -> Result<V::Value> where V: Visitor<'de>
    {
        if name == crate::value::value::MAGIC {
            let mut buf: Vec<u8> = Vec::new();
            crate::value::Encoder::new(&mut buf, None).write(self.input)
                .or_else(|_e| Err(Error::InternalMagicError))?;
            visitor.visit_byte_buf(buf)
        } else {
            let fs = self.check(|v| v.as_simple_record(name, Some(1)),
                                ExpectedKind::SimpleRecord(name, Some(1)))?;
            self.input = &fs[0];
            visitor.visit_newtype_struct(self)
        }
    }

    fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        let vs = self.check(|v| v.as_sequence(), ExpectedKind::Sequence)?;
        visitor.visit_seq(VecSeq::new(self, vs))
    }

    fn deserialize_tuple<V>(self, len: usize, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record("tuple", Some(len)),
                            ExpectedKind::SimpleRecord("tuple", Some(len)))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_tuple_struct<V>(self, name: &'static str, len: usize, visitor: V)
                                   -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record(name, Some(len)),
                            ExpectedKind::SimpleRecord(name, Some(len)))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_map<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        let d = self.check(|v| v.as_dictionary(), ExpectedKind::Dictionary)?;
        visitor.visit_map(DictMap::new(self, d))
    }

    fn deserialize_struct<V>(self,
                             name: &'static str,
                             fields: &'static [&'static str],
                             visitor: V)
                             -> Result<V::Value> where V: Visitor<'de>
    {
        let fs = self.check(|v| v.as_simple_record(name, Some(fields.len())),
                            ExpectedKind::SimpleRecord(name, Some(fields.len())))?;
        visitor.visit_seq(VecSeq::new(self, fs))
    }

    fn deserialize_enum<V>(self,
                           _name: &'static str,
                           _variants: &'static [&'static str],
                           visitor: V)
                           -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_enum(self)
    }

    fn deserialize_identifier<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_borrowed_str(&self.check(|v| v.as_symbol(), ExpectedKind::Symbol)?)
    }

    fn deserialize_ignored_any<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_none()
    }
}

pub struct VecSeq<'a, 'de: 'a, N: NestedValue> {
    index: usize,
    vec: &'de Vec<N>,
    de: &'a mut Deserializer<'de, N>,
}

impl<'de, 'a, N: NestedValue> VecSeq<'a, 'de, N> {
    fn new(de: &'a mut Deserializer<'de, N>, vec: &'de Vec<N>) -> Self {
        VecSeq{ index: 0, vec, de }
    }
}

impl<'de, 'a, N: NestedValue> SeqAccess<'de> for VecSeq<'a, 'de, N> {
    type Error = Error;

    fn next_element_seed<T>(&mut self, seed: T)
                            -> Result<Option<T::Value>> where T: DeserializeSeed<'de>
    {
        if self.index == self.vec.len() {
            return Ok(None)
        }

        self.de.input = &self.vec[self.index];
        self.index = self.index + 1;
        let value = seed.deserialize(&mut *self.de)?;
        Ok(Some(value))
    }
}

pub struct DictMap<'a, 'de: 'a, N: NestedValue> {
    pending: Option<&'de N>,
    iter: Box<dyn Iterator<Item = (&'de N, &'de N)> + 'a>,
    de: &'a mut Deserializer<'de, N>,
}

impl<'de, 'a, N: NestedValue> DictMap<'a, 'de, N> {
    fn new(de: &'a mut Deserializer<'de, N>, d: &'de Map<N, N>) -> Self {
        DictMap{ pending: None, iter: Box::new(d.into_iter()), de }
    }
}

impl<'de, 'a, N: NestedValue> MapAccess<'de> for DictMap<'a, 'de, N> {
    type Error = Error;

    fn next_key_seed<K>(&mut self, seed: K)
                        -> Result<Option<K::Value>> where K: DeserializeSeed<'de>
    {
        match self.iter.next() {
            None => Ok(None),
            Some((k, v)) => {
                self.pending = Some(v);
                self.de.input = k;
                Ok(Some(seed.deserialize(&mut *self.de)?))
            }
        }
    }

    fn next_value_seed<V>(&mut self, seed: V) -> Result<V::Value> where V: DeserializeSeed<'de> {
        let v = self.pending.unwrap();
        self.pending = None;
        self.de.input = v;
        Ok(seed.deserialize(&mut *self.de)?)
    }
}

impl<'a, 'de, N: NestedValue> EnumAccess<'de> for &'a mut Deserializer<'de, N> {
    type Error = Error;
    type Variant = Self;

    fn variant_seed<V>(self, seed: V)
                       -> Result<(V::Value, Self::Variant)> where V: DeserializeSeed<'de>
    {
        let (lp, _) = self.check(|v| v.as_record(), ExpectedKind::Record)?;
        let v = self.input;
        self.input = N::boxunwrap(lp);
        let variant = seed.deserialize(&mut *self)?;
        self.input = v;
        Ok((variant, self))
    }
}

impl<'a, 'de, N: NestedValue> VariantAccess<'de> for &'a mut Deserializer<'de, N> {
    type Error = Error;

    fn unit_variant(self) -> Result<()> {
        Ok(())
    }

    fn newtype_variant_seed<T>(self, seed: T) -> Result<T::Value> where T: DeserializeSeed<'de> {
        let (_, fs) = self.input.value().as_record().unwrap();
        self.input = &fs[0];
        seed.deserialize(&mut *self)
    }

    fn tuple_variant<V>(self, _len: usize, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
        visitor.visit_seq(VecSeq::new(self, &self.input.value().as_record().unwrap().1))
    }

    fn struct_variant<V>(self, _fields: &'static [&'static str], visitor: V)
                         -> Result<V::Value> where V: Visitor<'de>
    {
        visitor.visit_seq(VecSeq::new(self, &self.input.value().as_record().unwrap().1))
    }
}
